/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.admin;

import java.util.Iterator;
import java.util.Map;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.xml.transform.TransformerException;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.PetalsException;
import org.ow2.petals.communication.jmx.JMXService;
import org.ow2.petals.container.lifecycle.ComponentLifeCycle;
import org.ow2.petals.container.lifecycle.Installer;
import org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle;
import org.ow2.petals.jbi.management.MBeanNames;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.util.LoggingUtil;
import org.ow2.petals.util.ParameterCheckHelper;
import org.ow2.petals.util.XMLUtil;
import org.w3c.dom.Document;

/**
 * TODO AdminServiceImpl : check the exceptions catch
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author wjoseph - eBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.jbi.management.admin.AdminServiceMBean.class))
public class AdminServiceImpl implements AdminServiceMBean {

    /**
     * Logger wrapper
     */
    protected LoggingUtil log;

    /**
     * Monolog
     */
    @Monolog(name = "logger")
    private Logger logger;

    @Requires(name = "jmx", signature = org.ow2.petals.communication.jmx.JMXService.class)
    protected JMXService jmxService;

    /**
     * JBI :: JMX Agent Admin
     */
    protected LifeCycleManagerImpl manager;

    /**
     * Default constructor
     */
    public AdminServiceImpl() {
        super();
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#getBindingComponents()
     */
    public ObjectName[] getBindingComponents() {
        log.start();

        ObjectName[] names = new ObjectName[0];

        Map<ObjectName, ComponentLifeCycle> bindings = manager.getBindingCompoLifeCycles();

        names = bindings.keySet().toArray(names);

        return names;
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#getComponentByName(java.lang.String)
     */
    public ObjectName getComponentByName(String name) {
        log.call();
        ParameterCheckHelper.isNullParameterWithLog(name, "name must not be null", log);
        ParameterCheckHelper.isEmptyParameterWithLog(name, "name must not be empty", log);
        ObjectName on = null;

        // search in bindings

        on = getBindingComponentByName(name);

        if (on == null) {
            // search in engines

            on = getEngineComponentByName(name);
        }
        return on;
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#getEngineComponents()
     */
    public ObjectName[] getEngineComponents() {
        log.call();

        ObjectName[] names = new ObjectName[0];

        Map<ObjectName, ComponentLifeCycle> engines = manager.getEngineCompoLifeCycles();

        names = engines.keySet().toArray(names);

        return names;
    }

    /**
     * @see org.ow2.petals.jbi.management.admin.AdminServiceMBean#getServiceDescription(String,
     *      ServiceEndpoint)
     */
    public String getServiceDescription(String componentName, ServiceEndpoint endpoint)
            throws PetalsException {
        log.call();
        ComponentLifeCycle componentLifeCycle = getComponentLifeCycleByName(componentName);
        if (componentLifeCycle == null) {
            log.error("No such component " + componentName + " in the container");
            throw new PetalsException("No such component " + componentName + " in the container");
        }
        Document doc = componentLifeCycle.getComponent().getServiceDescription(endpoint);

        String result = "";
        if (doc != null) {
            try {
                result = XMLUtil.createStringFromDOMDocument(doc);
            } catch (TransformerException e) {
                String msg = "Cannot transform DOM endpoint description" + "into String";
                log.error(msg);
                throw new PetalsException(msg, e);
            }
        }
        return result;
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#getSystemInfo()
     */
    public String getSystemInfo() {
        return "Petals JBI Container - version: "
                + Package.getPackage("org.ow2.petals").getImplementationVersion();
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#getSystemService(java.lang.String)
     */
    public ObjectName getSystemService(String serviceName) {
        log.call();
        ParameterCheckHelper.isNullParameterWithLog(serviceName, "serviceName must not be null",
                log);
        ParameterCheckHelper.isEmptyParameterWithLog(serviceName, "serviceName must not be empty",
                log);
        ObjectName result = null;
        ObjectName[] systemServices = getSystemServices();

        for (int i = 0; i < systemServices.length && result == null; i++) {
            if (systemServices[i].getCanonicalName().equals(serviceName)) {
                result = systemServices[i];
            }
        }
        return result;
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#getSystemServices()
     */
    public ObjectName[] getSystemServices() {
        log.call();

        ObjectName[] names = new ObjectName[3];

        try {
            names[0] = getAdminServiceMBeanName();
            names[1] = getDeploymentServiceMBeanName();
            names[2] = getInstallationServiceMBeanName();
        } catch (ManagementException e) {
            log.warning("A system service name can't be retrieved !", e);
        }

        return names;
    }

    /**
     * @see javax.jbi.management.AdminServiceMBean#isBinding(java.lang.String)
     */
    public boolean isBinding(String componentName) {
        ParameterCheckHelper.isNullParameterWithLog(componentName,
                "componentName must not be null", log);
        ParameterCheckHelper.isEmptyParameterWithLog(componentName,
                "componentName must not be empty", log);
        boolean result = false;
        result = manager.getBindingCompoLifeCycles().containsKey(componentName);
        return result;
    }

    // ----------------------------------------------------------------------
    // Fractal BindingController implementation
    // ----------------------------------------------------------------------

    /**
     * @see javax.jbi.management.AdminServiceMBean#isEngine(java.lang.String)
     */
    public boolean isEngine(String componentName) {
        ParameterCheckHelper.isNullParameterWithLog(componentName,
                "componentName must not be null", log);
        ParameterCheckHelper.isEmptyParameterWithLog(componentName,
                "componentName must not be empty", log);
        boolean result = false;
        result = manager.getEngineCompoLifeCycles().containsKey(componentName);
        return result;
    }

    /**
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.management.admin.AdminServiceMBean#isExchangeWithConsumerOkayForComponent(java.lang.String,
     *      javax.jbi.servicedesc.ServiceEndpoint,
     *      javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithConsumerOkayForComponent(String componentName,
            ServiceEndpoint internalEndpoint, MessageExchange exchange) throws JBIException {
        log.call();
        ComponentLifeCycle componentLifeCycle = getComponentLifeCycleByName(componentName);
        if (componentLifeCycle == null) {
            log.error("No such component " + componentName + " in the container");
            throw new JBIException("No such component " + componentName + " in the container");
        }
        return componentLifeCycle.getComponent().isExchangeWithConsumerOkay(internalEndpoint,
                exchange);
    }

    /**
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.management.admin.AdminServiceMBean#isExchangeWithProviderOkayForComponent(java.lang.String,
     *      javax.jbi.servicedesc.ServiceEndpoint,
     *      javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithProviderOkayForComponent(String componentName,
            ServiceEndpoint internalEndpoint, MessageExchange exchange) throws JBIException {
        log.call();
        ComponentLifeCycle componentLifeCycle = getComponentLifeCycleByName(componentName);
        if (componentLifeCycle == null) {
            log.error("No such component " + componentName + " in the container");
            throw new JBIException("No such component " + componentName + " in the container");
        }
        return componentLifeCycle.getComponent().isExchangeWithProviderOkay(internalEndpoint,
                exchange);
    }

    // ----------------------------------------------------------------------
    // Fractal LifecycleController implementation
    // ----------------------------------------------------------------------

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    @LifeCycle(on = LifeCycleType.START)
    public void start() throws IllegalLifeCycleException {
        log = new LoggingUtil(logger);
        log.start();

        manager = new LifeCycleManagerImpl(jmxService.getLocalJMXServer(), log);

        log.end();
    }

    /**
     * Return the binding component associated with this name.
     * 
     * @param name
     * @return The related objectName, or null
     */
    protected ObjectName getBindingComponentByName(String name) {
        log.call();

        ObjectName result = null;

        Map<ObjectName, ComponentLifeCycle> bindings = manager.getBindingCompoLifeCycles();

        synchronized (bindings) {
            for (Iterator<ComponentLifeCycle> iter = bindings.values().iterator(); iter.hasNext()
                    && result == null;) {
                ComponentLifeCycle lifeCycle = (ComponentLifeCycle) iter.next();

                if (lifeCycle.getName().equals(name)) {
                    result = lifeCycle.getMBeanName();
                }
            }
        }
        return result;
    }

    /**
     * Return the engine component associated with this name.
     * 
     * @param name
     * @return The related objectName, or null
     */
    protected ObjectName getEngineComponentByName(String name) {
        log.call();

        ObjectName result = null;

        Map<ObjectName, ComponentLifeCycle> engines = manager.getEngineCompoLifeCycles();

        synchronized (engines) {
            for (Iterator<ComponentLifeCycle> iter = engines.values().iterator(); iter.hasNext()
                    && result == null;) {
                ComponentLifeCycle lifeCycle = (ComponentLifeCycle) iter.next();

                if (lifeCycle.getName().equals(name)) {
                    result = lifeCycle.getMBeanName();
                }
            }
        }
        return result;
    }

    // ///////////////////////////////
    // / methods delegated to LifeCycleManager
    // ///////////////////////////////

    public ObjectName getAdminServiceMBeanName() throws ManagementException {
        return manager.getAdminServiceMBeanName();
    }

    public Map<ObjectName, ComponentLifeCycle> getBindingCompoLifeCycles() {
        return manager.getBindingCompoLifeCycles();
    }

    public ComponentLifeCycle getComponentLifeCycleByName(String name) {
        return manager.getComponentLifeCycleByName(name);
    }

    public ObjectName getDeploymentServiceMBeanName() throws ManagementException {
        return manager.getDeploymentServiceMBeanName();
    }

    public Map<ObjectName, ComponentLifeCycle> getEngineCompoLifeCycles() {
        return manager.getEngineCompoLifeCycles();
    }

    public ObjectName getInstallationServiceMBeanName() throws ManagementException {
        return manager.getInstallationServiceMBeanName();
    }

    public ObjectName getInstallerByName(String name) {
        return manager.getInstallerByName(name);
    }

    public Installer getInstallerMBeanByName(ObjectName name) {
        return manager.getInstallerMBeanByName(name);
    }

    public Map<ObjectName, Installer> getInstallers() {
        return manager.getInstallers();
    }

    public MBeanNames getMBeanNames() {
        return manager.getMBeanNames();
    }

    public MBeanServer getMBeanServer() {
        return manager.getMBeanServer();
    }

    public ServiceAssemblyLifeCycle getServiceAssemblyByName(String saName) {
        return manager.getServiceAssemblyByName(saName);
    }

    public SharedLibraryLifeCycle getSharedLibraryByName(String slName) {
        return manager.getSharedLibraryByName(slName);
    }

    public Map<String, ServiceAssemblyLifeCycle> getServiceAssemblyLifeCycles() {
        return manager.getServiceAssemblyLifeCycles();
    }

    public Map<String, SharedLibraryLifeCycle> getSharedLibraryLifeCycles() {
        return manager.getSharedLibraryLifeCycles();
    }

    public ObjectName registerBindingComponent(ComponentLifeCycle lifeCycle)
            throws ManagementException {
        return manager.registerBindingComponent(lifeCycle);
    }

    public ObjectName registerEngineComponent(ComponentLifeCycle lifeCycle)
            throws ManagementException {
        return manager.registerEngineComponent(lifeCycle);
    }

    public ObjectName registerInstaller(Installer installerMBean) throws ManagementException {
        return manager.registerInstaller(installerMBean);
    }

    public void registerServiceAssembly(String saName,
            ServiceAssemblyLifeCycle serviceAssemblyLifeCycle) {
        manager.registerServiceAssembly(saName, serviceAssemblyLifeCycle);

    }

    public void unregisterBindingComponent(ObjectName name) throws ManagementException {
        manager.unregisterBindingComponent(name);

    }

    public void unregisterEngineComponent(ObjectName name) throws ManagementException {
        manager.unregisterEngineComponent(name);

    }

    public void unregisterInstaller(ObjectName name) throws ManagementException {
        manager.unregisterInstaller(name);

    }

    public void unregisterServiceAssembly(String saName) throws ManagementException {
        manager.unregisterServiceAssembly(saName);

    }

    public void registerSharedLibrary(String slName, SharedLibraryLifeCycle sharedLibraryLifeCycle) {
        manager.registerSharedLibrary(slName, sharedLibraryLifeCycle);
    }

    public void unregisterSharedLibrary(String slName) throws ManagementException {
        manager.unregisterSharedLibrary(slName);
    }

}
