/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DeploymentTaskFactory.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.management.task.deployment;

import org.ow2.petals.container.ContainerService;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.jbi.management.autoload.AutoLoaderService;
import org.ow2.petals.jbi.management.deployment.DeploymentServiceImpl;
import org.ow2.petals.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.jbi.management.task.CopyArchiveToInstalledDirTask;
import org.ow2.petals.jbi.management.task.CopyPackageToWorkDirTask;
import org.ow2.petals.jbi.management.task.DownloadTask;
import org.ow2.petals.jbi.management.task.ExtractTask;
import org.ow2.petals.jbi.management.task.SuccessTask;
import org.ow2.petals.jbi.management.task.deployment.deploy.CopySAPackageToRepositoryTask;
import org.ow2.petals.jbi.management.task.deployment.deploy.CreateAndRegisterSULifeCycleTask;
import org.ow2.petals.jbi.management.task.deployment.deploy.CreateSAStateHolderTask;
import org.ow2.petals.jbi.management.task.deployment.deploy.CreateXMLDeploymentResultTask;
import org.ow2.petals.jbi.management.task.deployment.deploy.DeployAllSUTask;
import org.ow2.petals.jbi.management.task.deployment.deploy.DeploySUToComponentask;
import org.ow2.petals.jbi.management.task.deployment.deploy.ExtractSUsTask;
import org.ow2.petals.jbi.management.task.deployment.deploy.RegisterAllConnectionTask;
import org.ow2.petals.jbi.management.task.deployment.deploy.RegisterSALifeCycleTask;
import org.ow2.petals.jbi.management.task.deployment.deploy.RenameSUsTask;
import org.ow2.petals.jbi.management.task.deployment.deploy.RetrieveSAAndSUInstallRootsTask;
import org.ow2.petals.jbi.management.task.deployment.deploy.SACheckPackageTask;
import org.ow2.petals.jbi.management.task.deployment.undeploy.CreateXMLUndeploymentResultTask;
import org.ow2.petals.jbi.management.task.deployment.undeploy.DeleteArchiveFromInstalledDirTask;
import org.ow2.petals.jbi.management.task.deployment.undeploy.RemoveAllConnectionsTask;
import org.ow2.petals.jbi.management.task.deployment.undeploy.RemoveSAFolderTask;
import org.ow2.petals.jbi.management.task.deployment.undeploy.RemoveSAStateHolderTask;
import org.ow2.petals.jbi.management.task.deployment.undeploy.RetrieveSALifeCycleTask;
import org.ow2.petals.jbi.management.task.deployment.undeploy.UndeployAllSUsTask;
import org.ow2.petals.jbi.management.task.deployment.undeploy.UnregisterSALifeCycleTask;
import org.ow2.petals.jbi.management.util.PackageHandler;
import org.ow2.petals.platform.repository.RepositoryService;
import org.ow2.petals.platform.systemstate.SystemStateService;
import org.ow2.petals.util.LoggingUtil;

/**
 * This Factory creates Deployment Tasks.
 *
 * @author ofabre - EBM Websourcing
 *
 */
public class DeploymentTaskFactory {

    protected CreateXMLDeploymentResultTask createXMLDeploymentResultTask;

    protected SACheckPackageTask saCheckPackageTask;

    protected ExtractSUsTask extractSUsTask;

    protected DeployAllSUTask deployAllSUTask;

    protected RegisterSALifeCycleTask registerSALifeCycleTask;

    protected RegisterAllConnectionTask registerAllConnectionTask;

    protected CreateSAStateHolderTask createSAStateHolderTask;

    protected CopyArchiveToInstalledDirTask copyArchiveToInstalledDirTask;

    protected DeploySUToComponentask deploySUToComponentTask;

    protected CreateAndRegisterSULifeCycleTask createAndRegisterSULifeCycleTask;

    protected SuccessTask successTask;

    protected RetrieveSAAndSUInstallRootsTask retrieveSAAndSUInstallRootsTask;

    protected RemoveAllConnectionsTask removeAllConnectionsTask;

    protected UndeployAllSUsTask undeployAllSUsTask;

    protected RemoveSAFolderTask removeSAFolderTask;

    protected DeleteArchiveFromInstalledDirTask deleteArchiveFromInstalledDirTask;

    protected RetrieveSALifeCycleTask retrieveSALifeCycleTask;

    protected UnregisterSALifeCycleTask unregisterSALifeCycleTask;

    protected RemoveSAStateHolderTask removeSAStateHolderTask;

    protected CreateXMLUndeploymentResultTask createXMLUndeploymentResultTask;

    private CopySAPackageToRepositoryTask copySAPackageToRepositoryTask;

    private ExtractTask extractTask;

    private CopyPackageToWorkDirTask copyPackageToWorkDirTask;

    private RenameSUsTask renameSUsTask;

    private DownloadTask downloadTask;

    public DeploymentTaskFactory(PackageHandler packageHandler,
        EndpointRegistry EndpointRegistry,
        DeploymentServiceImpl deploymentService,
        SystemStateService systemStateService,
        RepositoryService repositoryService, LoggingUtil log,
        AdminServiceMBean adminService, ContainerService containerService,
        AutoLoaderService autoLoaderService) {

        /*
         * Create Tasks
         */

        createXMLDeploymentResultTask = new CreateXMLDeploymentResultTask();

        saCheckPackageTask = new SACheckPackageTask(log, packageHandler,
            systemStateService, adminService);

        extractSUsTask = new ExtractSUsTask(packageHandler, repositoryService,
            log);

        registerSALifeCycleTask = new RegisterSALifeCycleTask(adminService,
            log, containerService);

        deployAllSUTask = new DeployAllSUTask(this, log);

        registerAllConnectionTask = new RegisterAllConnectionTask(
            EndpointRegistry, log);

        createSAStateHolderTask = new CreateSAStateHolderTask(
            systemStateService, repositoryService, autoLoaderService, log);

        copyArchiveToInstalledDirTask = new CopyArchiveToInstalledDirTask(autoLoaderService, log);

        deploySUToComponentTask = new DeploySUToComponentask(log, adminService);

        createAndRegisterSULifeCycleTask = new CreateAndRegisterSULifeCycleTask(
            log, adminService);

        successTask = new SuccessTask(autoLoaderService, log);

        retrieveSAAndSUInstallRootsTask = new RetrieveSAAndSUInstallRootsTask(
            repositoryService, packageHandler);

        removeAllConnectionsTask = new RemoveAllConnectionsTask(EndpointRegistry);

        undeployAllSUsTask = new UndeployAllSUsTask(adminService);

        removeSAFolderTask = new RemoveSAFolderTask(repositoryService);

        retrieveSALifeCycleTask = new RetrieveSALifeCycleTask(adminService);

        unregisterSALifeCycleTask = new UnregisterSALifeCycleTask(adminService,
            containerService);

        removeSAStateHolderTask = new RemoveSAStateHolderTask(
            systemStateService);

        createXMLUndeploymentResultTask = new CreateXMLUndeploymentResultTask();

        deleteArchiveFromInstalledDirTask = new DeleteArchiveFromInstalledDirTask(autoLoaderService);

        copySAPackageToRepositoryTask = new CopySAPackageToRepositoryTask(
            repositoryService, log);

        extractTask = new ExtractTask(autoLoaderService, log);

        copyPackageToWorkDirTask = new CopyPackageToWorkDirTask(autoLoaderService, log);

        renameSUsTask = new RenameSUsTask(packageHandler, repositoryService,
            log);

        downloadTask = new DownloadTask(autoLoaderService, log);
    }

    public DownloadTask getDownloadTask() {
        return downloadTask;
    }

    public RenameSUsTask getRenameSUsTask() {
        return renameSUsTask;
    }

    public RegisterAllConnectionTask getRegisterAllConnectionTask() {
        return registerAllConnectionTask;
    }

    public DeployAllSUTask getDeployAllSUTask() {
        return deployAllSUTask;
    }

    public CopyArchiveToInstalledDirTask getCopyArchiveToInstalledDirTask() {
        return copyArchiveToInstalledDirTask;
    }

    public CreateAndRegisterSULifeCycleTask getCreateAndRegisterSULifeCycleTask() {
        return createAndRegisterSULifeCycleTask;
    }

    public ExtractSUsTask getExtractSUsTask() {
        return extractSUsTask;
    }

    public SACheckPackageTask getSaCheckPackageTask() {
        return saCheckPackageTask;
    }

    public CreateXMLDeploymentResultTask getCreateXMLDeploymentResultTask() {
        return createXMLDeploymentResultTask;
    }

    public RegisterSALifeCycleTask getRegisterSALifeCycleTask() {
        return registerSALifeCycleTask;
    }

    public CreateSAStateHolderTask getCreateSAStateHolderTask() {
        return createSAStateHolderTask;
    }

    public DeploySUToComponentask getDeploySUToComponentTask() {
        return deploySUToComponentTask;
    }

    public RemoveAllConnectionsTask getRemoveAllConnectionsTask() {
        return removeAllConnectionsTask;
    }

    public UndeployAllSUsTask getUndeployAllSUTask() {
        return undeployAllSUsTask;
    }

    public RemoveSAFolderTask getRemoveSAFolderTask() {
        return removeSAFolderTask;
    }

    public RetrieveSALifeCycleTask getRetrieveSALifeCycleTask() {
        return retrieveSALifeCycleTask;
    }

    public UnregisterSALifeCycleTask getUnregisterSALifeCycleTask() {
        return unregisterSALifeCycleTask;
    }

    public RemoveSAStateHolderTask getRemoveSAStateHolderTask() {
        return removeSAStateHolderTask;
    }

    public CreateXMLUndeploymentResultTask getCreateXMLUndeploymentResultTask() {
        return createXMLUndeploymentResultTask;
    }

    public RetrieveSAAndSUInstallRootsTask getRetrieveSAAndSUInstallRootsTask() {
        return retrieveSAAndSUInstallRootsTask;
    }

    public DeleteArchiveFromInstalledDirTask getDeleteArchiveFromInstalledDirTask() {
        return deleteArchiveFromInstalledDirTask;
    }

    public SuccessTask getSuccessTask() {
        return successTask;
    }

    public CopySAPackageToRepositoryTask getCopySAPackageToRepositoryTask() {
        return copySAPackageToRepositoryTask;
    }

    public ExtractTask getExtractTask() {
        return extractTask;
    }

    public CopyPackageToWorkDirTask getCopyArchiveToWorkDirTask() {
        return copyPackageToWorkDirTask;
    }

}
