/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: RegisterSALifeCycleTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.management.task.deployment.deploy;

import java.net.URI;
import java.util.HashMap;

import javax.management.ObjectName;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.ow2.petals.PetalsException;
import org.ow2.petals.container.ContainerService;
import org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.jbi.descriptor.JBIDescriptor;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.jbi.management.task.deployment.DeploymentContextConstants;
import org.ow2.petals.jbi.management.task.deployment.DeploymentUtils;
import org.ow2.petals.kernel.server.FractalHelper;
import org.ow2.petals.processor.Task;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task registers ServiceAssemblyLifeCycle into manager service map <br/>
 * Inputs : {@link JBIDescriptor}, {@link ServiceAssemblyLifeCycle} <br/>
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class RegisterSALifeCycleTask implements Task {

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected AdminServiceMBean adminService;
    
    protected ContainerService containerService;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    public RegisterSALifeCycleTask(AdminServiceMBean adminService,
        LoggingUtil log, ContainerService containerService) {
        super();
        this.adminService = adminService;
        this.containerService = containerService;
        this.log = log;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        URI installationRoot = (URI) context
        .get(TaskContextConstants.INSTALL_ROOT);
        
        JBIDescriptor descriptor = (JBIDescriptor) context
            .get(TaskContextConstants.DESCRIPTOR);
        
        /*
         * Create service assembly lifecycle element
         */
        ServiceAssemblyLifeCycle saLifeCycle = createSALifeCycle(
            installationRoot, descriptor);

        adminService.registerServiceAssembly(DeploymentUtils
            .getServiceAssemblyName(descriptor), saLifeCycle);
        
        /*
         * Fill Context
         */
        context.put(DeploymentContextConstants.SA_LIFECYCLE, saLifeCycle);
    }
    
    /**
     * Creates a new {@link ServiceAssemblyLifeCycle}
     * 
     * @param installationRoot
     *            the root {@link URI} of the service assembly into the
     *            repository
     * @param descriptor
     *            the service assembly jbi descriptor
     * @return a new {@link ServiceAssemblyLifeCycle}
     * @throws ManagementException
     *             if deployment service MBean name retrieval fails
     * @throws PetalsException 
     * @throws ADLException 
     * @throws IllegalLifeCycleException 
     * @throws NoSuchInterfaceException 
     */
    protected ServiceAssemblyLifeCycle createSALifeCycle(URI installationRoot,
        JBIDescriptor descriptor) throws ManagementException, PetalsException, NoSuchInterfaceException, IllegalLifeCycleException, ADLException {
        // Retrieve the DeploymentServiceImpl ObjectName
        ObjectName objectName = adminService.getDeploymentServiceMBeanName();

        Component serviceAssemblyLifeCycleComponent = containerService.createServiceAssemblyLifeCycle(descriptor.getServiceAssembly());
        
        FractalHelper.getFractalHelper().startComponent(serviceAssemblyLifeCycleComponent);

        ServiceAssemblyLifeCycle serviceAssemblyLifeCycle = (ServiceAssemblyLifeCycle)serviceAssemblyLifeCycleComponent.getFcInterface("/content");
        
        serviceAssemblyLifeCycle.init(objectName, descriptor
                        .getServiceAssembly(), installationRoot);

        return serviceAssemblyLifeCycle;

    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) {
        JBIDescriptor descriptor = (JBIDescriptor) context
            .get(TaskContextConstants.DESCRIPTOR);

        try {
            adminService.unregisterServiceAssembly(DeploymentUtils
                .getServiceAssemblyName(descriptor));
        } catch (ManagementException e) {
            String msg = "Failed to revert a RegisterSALifeCycleTask: ";
            log.error(msg + e.getMessage());
        }
    }

}
