/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: EndpointRegistryMBean.java 11:27:27 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.messaging.registry;

import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.w3c.dom.Document;

/**
 * EndpointRegistryMBean interface exposes method to retrieve information about
 * endpoints activated in the JBI environment for a single Domain (Joram Domain)
 *
 * @author ofabre - eBM Websourcing
 *
 */
public interface EndpointRegistryMBean {

    /**
     * Retrieve all activated internal endpoints information for the current
     * Joram Domain. An endpoint is mapped to a {@link Map} with the following
     * fields : <TABLE>
     * <TR>
     * <TH>key</TH>
     * <TH>type</TH>
     * </TR>
     * <TR>
     * <TD>componentName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>containerName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>endpointName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>serviceName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>interfaceNames</TD>
     * <TD>String[]</TD>
     * </TR>
     * </TABLE>
     *
     *
     * @return a {@link List} of all activated internal endpoints information
     *         for the current Joram Domain. Must be non null, may be empty
     */
    List<Map<String, Object>> getAllInternalEndpoints();

    /**
     * Retrieve all activated external endpoints information for the current
     * Joram Domain. An endpoint is mapped to a {@link Map} with the following
     * fields : <TABLE>
     * <TR>
     * <TH>key</TH>
     * <TH>type</TH>
     * </TR>
     * <TR>
     * <TD>componentName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>containerName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>endpointName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>serviceName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>interfaceNames</TD>
     * <TD>String[]</TD>
     * </TR>
     * </TABLE>
     *
     *
     * @return a {@link List} of all activated external endpoints information
     *         for the current Joram Domain. Must be non null, may be empty
     */
    List<Map<String, Object>> getAllExternalEndpoints();

    /**
     * Retrieve all activated endpoints (internal, external and connections)
     * information for the current Joram Domain. An endpoint is mapped to a
     * {@link Map} with the following fields : <TABLE>
     * <TR>
     * <TH>key</TH>
     * <TH>type</TH>
     * </TR>
     * <TR>
     * <TD>componentName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>containerName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>endpointName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>serviceName</TD>
     * <TD>String</TD>
     * </TR>
     * <TR>
     * <TD>interfaceNames</TD>
     * <TD>String[]</TD>
     * </TR>
     * <TR>
     * <TD>type</TD>
     * <TD>String</TD>
     * </TR>
     * </TABLE>
     *
     * The type field follow this pattern : 'internal' if it's an internal
     * endpoint, 'external' if it's an external endpoint or
     * '{serviceNamespace}serviceLocalpart@endpointName' if it's a connection
     *
     * @return a {@link List} of all activated endpoints (internal, external and
     *         connections) information for the current Joram Domain. Must be
     *         non null, may be empty
     */
    List<Map<String, Object>> getAllEndpoints();

    /**
     * Returns a Dom representation of a ServiceEndpoint description
     *
     * @param serviceName
     *            the service {@link QName} of this endpoint
     * @param endpointName
     *            the unique name of this endpoint
     * @return a Dom representation of a ServiceEndpoint description
     * @throws Exception
     *             if an error occurs during endpoint description retrieval
     */
    Document getDescription(String serviceName, String endpointName) throws Exception;

}
