/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.messaging.routing.EndpointChooser;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.ow2.petals.jbi.messaging.routing.RoutingException;

/**
 * @author Roland NAUDIN - EBM WebSourcing
 * 
 */
public class HighestStrategy implements Strategy {

    private static final int NUMBER_STRATEGY_PARAMETERS = 3;

    public EndpointChooserStrategy endpointChooserStrategy;

    private float localWeighting;

    private float remoteActiveWeighting;

    private float remoteInactiveWeighting;

    public HighestStrategy(EndpointChooserStrategy endpointChooserStrategy) {
        this.endpointChooserStrategy = endpointChooserStrategy;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.EndpointChooser.Strategy#validateStrategyParameters(java.util.List)
     */
    public void validateStrategyParameters(List<String> strategyParameters) throws RoutingException {

        try {
            if (strategyParameters.size() == NUMBER_STRATEGY_PARAMETERS) {
                Iterator<String> it = strategyParameters.iterator();
                localWeighting = Float.valueOf(it.next());
                remoteActiveWeighting = Float.valueOf(it.next());
                remoteInactiveWeighting = Float.valueOf(it.next());
            } else {
                throw new RoutingException(
                        "Invalid parameters: 3 parameters are required by the highest routing strategy");
            }
        } catch (NumberFormatException e) {
            throw new RoutingException(
                    "Impossible to convert values of parameters for the highest routing strategy");
        }

        if (localWeighting < 0 || remoteActiveWeighting < 0 || remoteInactiveWeighting < 0) {
            throw new RoutingException("Negative parameter not allowed by the highest strategy");
        }

        if (localWeighting == 0 && remoteActiveWeighting == 0 && remoteInactiveWeighting == 0) {
            throw new RoutingException("Weighting 0,0,0 is not allowed by the highest strategy");
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.EndpointChooser.Strategy#electEndpoint(java.util.List,
     *      javax.jbi.messaging.MessageExchange)
     */
    public ServiceEndpoint electEndpoint(List<ServiceEndpoint> endpoints, MessageExchange exchange) throws RoutingException {
        if (exchange == null) {
            throw new NullPointerException("The exchange cannot be null");
        }

        ServiceEndpoint result = null;
        Float maxValue = new Float(0);

        if (endpoints != null && endpoints.size() > 0) {

            Map<ServiceEndpoint, Float> weightingEndpointsMap = endpointChooserStrategy
                    .getWeightingEndpoints(endpoints, localWeighting, remoteActiveWeighting,
                            remoteInactiveWeighting);

            for (ServiceEndpoint endpoint : weightingEndpointsMap.keySet()) {
                if (weightingEndpointsMap.get(endpoint).compareTo(maxValue) >= 0) {
                    result = endpoint;
                    maxValue = weightingEndpointsMap.get(endpoint);
                }
            }
        }
        return result;
    }
    
}
