/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsAdminServiceMBean.java 16:40:52 alouis $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.kernel.admin;

import java.util.Map;
import java.util.Set;

import org.ow2.petals.PetalsException;
import org.ow2.petals.tools.jmx.api.IPetalsAdminService;

/**
 * This class is a manager of JMX server. It is used to get a connection to a
 * JMX Server on a distant container.
 *
 * @author ddesjardins - eBMWebsourcing
 */
public interface PetalsAdminServiceMBean extends IPetalsAdminService {

    /**
     * Domain keys
     */
    public static final String CONF_DOMAIN_NAME = DomainKeyNames.CONF_DOMAIN_NAME;

    public static final String CONF_DOMAIN_MODE = DomainKeyNames.CONF_DOMAIN_MODE;

    public static final String CONF_NETWORK_MULTICAST_IP = DomainKeyNames.CONF_NETWORK_MULTICAST_IP;

    public static final String CONF_NETWORK_MULTICAST_PORT = DomainKeyNames.CONF_NETWORK_MULTICAST_PORT;

    public static final String CONF_JNDI_FACTORY = DomainKeyNames.CONF_JNDI_FACTORY;

    public static final String CONF_JNDI_HOST = DomainKeyNames.CONF_JNDI_HOST;

    public static final String CONF_JNDI_PORT = DomainKeyNames.CONF_JNDI_PORT;

    /**
     * Container keys
     */
    public static final String CONF_CONTAINER_NAME = ContainerKeyNames.CONF_CONTAINER_NAME;

    public static final String CONF_SUBDOMAIN_NAME = ContainerKeyNames.CONF_SUBDOMAIN_NAME;

    public static final String CONF_HOST = ContainerKeyNames.CONF_HOST;

    public static final String CONF_NETWORK_BIND_PORT = ContainerKeyNames.CONF_NETWORK_BIND_PORT;

    public static final String CONF_USER = ContainerKeyNames.CONF_USER;

    public static final String CONF_PWD = ContainerKeyNames.CONF_PWD;

    public static final String CONF_JMX_RMI_PORT = ContainerKeyNames.CONF_JMX_RMI_PORT;

    public static final String CONF_JMX_JNDI_PORT = ContainerKeyNames.CONF_JMX_JNDI_PORT;

    public static final String CONF_JORAM_DOMAIN_PORT = ContainerKeyNames.CONF_JORAM_DOMAIN_PORT;

    public static final String CONF_JORAM_TCP_PORT = ContainerKeyNames.CONF_JORAM_TCP_PORT;

    public static final String CONF_DREAM_TCP_PORT = ContainerKeyNames.CONF_DREAM_TCP_PORT;

    public static final String CONF_STATE = ContainerKeyNames.CONF_STATE;

    public static final String STATE_STARTED = ContainerStates.STATE_STARTED;

    public static final String STATE_STOPPED = ContainerStates.STATE_STOPPED;

    /**
     * Retrieve the collection of all the servers configuration.
     *
     * @return a set of container configuration in key/value form
     * @throws PetalsException
     */
    Set<Map<String, String>> retrieveContainersConfiguration() throws PetalsException;

    /**
     * Retrieve the collection of all the Started servers configuration.
     *
     * @return a set of container configuration in key/value form
     * @throws PetalsException
     */
    Set<Map<String, String>> retrieveStartedContainersConfiguration() throws PetalsException;

    /**
     * Unload the container from the Petals global environment. All
     * ServiceAssemblies, Components and Share Libraries are stopped, shutdown,
     * and uninstalled. All registered Endpoints are deactivated. Then the
     * container is removed from the domain.
     *
     * @throws Exception
     */
    void shutdownContainer() throws Exception;

    /**
     * Stop the container. The container is still visible in the Petals global
     * environment, but marked as 'stopped'. The registered Endpoints are still
     * visible,too.
     *
     * @throws Exception
     */
    void stopContainer() throws Exception;
}
