/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.kernel.interceptors.stat;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.TimerTask;

import org.ow2.petals.util.SystemUtil;

/**
 * This WriteStatsTask is a service that periodically write the statisstics of
 * methods in thae associated class file. in function of given period.
 *
 * @author Nicolas SALATGE - eBMWebSourcing
 *
 */
public class WriteStatsTask extends TimerTask {

    /**
     * Stat file name.
     */
    private static final String STAT_FILE_NAME = "stats.csv";

    /**
     * the file writer of stat file.
     */
    private static FileWriter statFileWriter = null;

    /**
     * the fractal controller.
     */
    private StatControllerImpl controller;

    /**
     * the default constructor.
     *
     * @param c
     *            the controller
     */
    public WriteStatsTask(final StatControllerImpl c) {
        this.controller = c;
        final String rootPath = SystemUtil.getPetalsInstallDirectory().getName();
        if (WriteStatsTask.statFileWriter == null) {
            final String log = rootPath + File.separator + "logs"
                    + File.separator + WriteStatsTask.STAT_FILE_NAME;
            try {
                WriteStatsTask.statFileWriter = new FileWriter(log, true);
            } catch (final IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * run the task.
     */
    @Override
    public void run() {
        if (WriteStatsTask.statFileWriter != null) {
            this.write();
        }
    }

    /**
     * write in file.
     *
     */
    protected void write() {
        try {
            StringTokenizer st = null;
            String interfaceName = null;
            String methodName = null;
            for (final InfoStat stat : this.controller.getStats()) {
                st = new StringTokenizer(stat.getMethodName(), "_");
                if (st.countTokens() >= 2) {
                    interfaceName = st.nextToken();
                    methodName = st.nextToken();
                } else {
                    throw new IOException(
                            "Impossible to analize the method name");
                }
                WriteStatsTask.statFileWriter.append(stat.getBeginDateInLong() + "\t"
                        + stat.getDate() + "\t" + interfaceName + "\t"
                        + methodName + "\t" + stat.getThreadName() + "\t"
                        + stat.getTime() + "\n");
                WriteStatsTask.statFileWriter.flush();
            }
            this.controller.getStats().clear();

        } catch (final IOException e) {
        }
    }

}
