/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.kernel.server;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.objectweb.dream.protocol.channel.TCPIPProtocolImplAttributeController;
import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.util.Fractal;
import org.ow2.petals.PetalsException;

/**
 * This class helps fractal management.
 *
 * @author alouis, rnaudin
 *
 */
public class FractalHelper {

    /**
     * Fractal Petals names.
     */
    public static final String PETALS_COMPOSITE = "Petals";

    /**
     * The Communication component name.
     */
    public static final String COMM_COMPOSITE = "Communication";

    /**
     * The configuration service component name.
     */
    public static final String CONFIGURATION_COMPONENT = "ConfigurationServiceImpl";

    /**
     * The network service component name.
     */
    public static final String NETWORK_COMPONENT = "NetworkServiceImpl";

    /**
     * The topology service component name.
     */
    public static final String TOPOLOGY_COMPONENT = "TopologyServiceImpl";

    /**
     * The JNDI agent service component name.
     */
    public static final String JNDIAGENT_COMPONENT = "JNDIAgentServiceImpl";

    /**
     * The Jndi service component name.
     */
    public static final String JNDI_COMPONENT = "JNDIServiceImpl";

    /**
     * The JMX service component name.
     */
    public static final String JMX_COMPONENT = "JMXServiceImpl";

    /**
     * The JBI Management component name.
     */
    public static final String JBI_MANAGEMENT_COMPOSITE = "JBI-Management";

    /**
     * The Admin Service component name.
     */
    public static final String ADMIN_COMPONENT = "AdminServiceImpl";

    /**
     * The deployment component name.
     */
    public static final String DEPLOYMENT_COMPONENT = "DeploymentServiceImpl";

    /**
     * The installation component name.
     */
    public static final String INSTALLATION_COMPONENT = "InstallationServiceImpl";

    /**
     * The endpoint service component name.
     */
    public static final String ENDPOINT_COMPONENT = "EndpointRegistryImpl";

    /**
     * The system recovery component name.
     */
    public static final String SYSTEMRECOVERY_COMPONENT = "SystemRecoveryServiceImpl";

    /**
     * The autoloader component name.
     */
    public static final String AUTOLOADER_COMPONENT = "AutoLoaderServiceImpl";

    /**
     * the JBI messaging composite name.
     */
    public static final String JBI_MESSAGING_COMPOSITE = "JBI-Messaging";

    public static final String TRANSPORTER_COMPOSITE = "Transporter";

    public static final String STANDALONE_TRANSPORTER_COMPONENT = "StandaloneTransporter";

    public static final String DISPATCHER_COMPONENT = "Dispatcher";

    public static final String TRANSPORTMONITORING_COMPONENT = "TransportMonitoring";

    /**
     * the transporters component name.
     */
    public static final String JORAMTRANSPORTER_COMPONENT = "JoramTransporter";

    public static final String MEMORYTRANSPORTER_COMPONENT = "MemoryTransporter";

    public static final String DREAMTRANSPORTER_COMPONENT = "DreamTransporter";

    public static final String DREAMTRANSPORTER_COMPOSITE = "DreamTransportProtocol";

    /**
     * The Petals admin service component name.
     */
    public static final String PETALSADMIN_COMPONENT = "PetalsAdminServiceImpl";

    /**
     * the explorer composite component name.
     */
    public static final String EXPLORER_COMPOSITE = "Explorer";

    /**
     * the petals component name.
     */
    public static final String PETALS_EXPLORER_COMPOSITE = "PetalsExplorer";

    public static final String ROUTER_MONITOR_COMPONENT = "RouterMonitorImpl";

    /**
     * the FCAPPL binding name.
     */
    public static final String FCAPPL_BINDING = "fcAppl";

    /**
     * The singleton internal instance.
     */
    private static FractalHelper fractalHelper;

    /**
     * the context.
     */
    private static Map<String, Object> context = null;

    /**
     * the fractal factory.
     */
    private static Factory factory = null;

    /**
     * Create a FractalHelper instance.
     *
     * @throws ADLException :
     *             occurs when it is impossible to get the fractal explorer
     */
    private FractalHelper() throws ADLException {
        super();

        // check required fractal properties
        System.setProperty("fractal.provider", "org.objectweb.fractal.julia.Julia");
        System.setProperty("julia.loader", "org.objectweb.fractal.julia.loader.DynamicLoader");
        System.setProperty("julia.config", "julia.cfg,julia-dream.cfg,julia-dream-activities.cfg");

        // Create Fractal ADL factories
        FractalHelper.context = new HashMap<String, Object>();
        FractalHelper.factory = FactoryFactory.getFactory(FactoryFactory.FRACTAL_BACKEND,
                FractalHelper.context);
    }

    /**
     * Get a FractalHelper instance.
     *
     * @return the fractalHelper instance
     * @throws ADLException :
     *             occurs when it is impossible to get the fractal explorer
     */
    public static FractalHelper getFractalHelper() throws ADLException {
        if (FractalHelper.fractalHelper == null) {
            FractalHelper.fractalHelper = new FractalHelper();
        }
        return FractalHelper.fractalHelper;
    }

    /**
     * Get the factory to manage components.
     *
     * @return The factory
     * @throws ADLException :
     *             occurs when the framework of Petals is incorrect
     */
    public Factory getFactory() throws ADLException {
        if (FractalHelper.factory == null) {
            new FractalHelper();
        }
        return FractalHelper.factory;
    }

    /**
     * Create a new component.
     *
     * @param name
     *            The class name of the component
     * @return the new component
     * @throws ADLException :
     *             occurs when the framework of Petals is incorrect
     */
    public Component createNewComponent(final String name) throws ADLException {
        return (Component) this.getFactory().newComponent(name, FractalHelper.context);
    }

    /**
     * A utility function allowing to get a component from any content
     * controller.
     *
     * @param parentContentController
     *            parentContentController
     * @param name
     *            component name
     * @return the component, null if not found
     */
    public Component getComponentByName(final ContentController parentContentController,
            final String name) {

        // the component to be returned
        Component matchedComponent = null;

        // the subcomponent content controller
        ContentController subContentController = null;

        // List content controller subcomponents
        for (Component component : parentContentController.getFcSubComponents()) {

            // if the component is a composite, search the matching component
            // recursively
            try {
                subContentController = Fractal.getContentController(component);
                if (subContentController.getFcSubComponents().length > 0) {
                    matchedComponent = this.getComponentByName(subContentController, name);
                    if (matchedComponent != null) {
                        break;
                    }
                }
            } catch (NoSuchInterfaceException e1) {
                // do nothing
                matchedComponent = null;
            }

            try {
                String componentName = Fractal.getNameController(component).getFcName();
                if (componentName.equals(name)) {
                    matchedComponent = component;
                    break;
                }
            } catch (NoSuchInterfaceException e) {
                // do nothing, return null
                matchedComponent = null;
            }
        }
        return matchedComponent;
    }

    /**
     * A utility function allowing to get a component LifeCycleController from a
     * content controller.
     *
     * @param parentContentController
     *            the parent content controller
     * @param name
     *            the name of the component
     * @return the LifeCycleController of the component, null if not found
     */
    protected LifeCycleController getLifeCycleControllerByName(
            final ContentController parentContentController, final String name) {

        LifeCycleController lifeCycleController = null;

        Component comp = this.getComponentByName(parentContentController, name);

        if (comp != null) {
            try {
                lifeCycleController = Fractal.getLifeCycleController(comp);
            } catch (NoSuchInterfaceException e) {
                // do nothing, return null
                lifeCycleController = null;
            }
        }
        return lifeCycleController;
    }

    /**
     * A utility function to start the given fractal component.
     *
     * @param component
     *            the fractal component
     *
     * @return true if the component was found and stopped, false otherwise
     * @throws NoSuchInterfaceException :
     *             impossible to stop the component
     * @throws IllegalLifeCycleException :
     *             impossible to stop the component
     */
    public boolean startComponent(final Component component) throws NoSuchInterfaceException,
            IllegalLifeCycleException {
        boolean result = false;

        LifeCycleController lifeCycleController = Fractal.getLifeCycleController(component);

        if (lifeCycleController != null) {
            if (LifeCycleController.STOPPED.equals(lifeCycleController.getFcState())) {
                lifeCycleController.startFc();
                result = true;
            }
        }

        return result;
    }

    /**
     * A utility function to stop the given fractal component.
     *
     * @param component
     *            the fractal component to stop
     *
     * @return true if the component was found and stopped, false otherwise
     * @throws NoSuchInterfaceException :
     *             impossible to stop the component
     * @throws IllegalLifeCycleException :
     *             impossible to stop the component
     */
    public boolean stopComponent(final Component component) throws NoSuchInterfaceException,
            IllegalLifeCycleException {

        boolean result = false;
        LifeCycleController lifeCycleController = Fractal.getLifeCycleController(component);
        if (LifeCycleController.STARTED.equals(lifeCycleController.getFcState())) {
            lifeCycleController.stopFc();
            result = true;
        }
        return result;
    }

    /**
     * A utility function to stop the given fractal composite. TODO: Must be
     * remove when the bug no JBOSS environment would be resolved.
     *
     * @param composite
     * @throws NoSuchInterfaceException
     * @throws IllegalLifeCycleException
     * @throws NoSuchInterfaceException
     */
    public void stopComposite(final Component composite) throws IllegalLifeCycleException,
            NoSuchInterfaceException {
        // the subcomponent content controller
        ContentController subContentController = null;
        ContentController parentContentController = Fractal.getContentController(composite);

        // List content controller sub components
        for (int i = parentContentController.getFcSubComponents().length; i != 0; i--) {
            Component component = parentContentController.getFcSubComponents()[i - 1];
            // if the component is a composite, stop the composite
            // recursively
            try {
                subContentController = Fractal.getContentController(component);
                if (subContentController.getFcSubComponents().length > 0) {
                    stopComposite(component);
                }
            } catch (NoSuchInterfaceException e) {
                // stop the component
                LifeCycleController lifeCycleController = Fractal.getLifeCycleController(component);
                if (LifeCycleController.STARTED.equals(lifeCycleController.getFcState())) {
                    lifeCycleController.stopFc();
                }
            }
        }
    }

    /**
     * Add a component in a composite component.
     *
     * @param newComponent
     *            The new component
     * @param parentComponent
     *            The parent component
     * @param listOfBindings
     *            the list of binding to create between this new component and
     *            the others
     * @throws NoSuchInterfaceException :
     *             Impossible to add a component
     * @throws IllegalLifeCycleException :
     *             Impossible to add a component
     * @throws IllegalContentException :
     *             Impossible to add a component
     * @throws IllegalBindingException :
     *             Impossible to add a component
     */
    public void addComponent(final Component newComponent, final Component parentComponent,
            final List<Binding> listOfBindings) throws NoSuchInterfaceException,
            IllegalContentException, IllegalLifeCycleException, IllegalBindingException {
        // Get the parent controller
        ContentController parentContentController = Fractal.getContentController(parentComponent);

        parentContentController.addFcSubComponent(newComponent);

        // Get the binding controller of the new component
        BindingController cBindingController = Fractal.getBindingController(newComponent);

        // Add all the bindings
        if (listOfBindings != null) {
            for (int i = 0; i < listOfBindings.size(); i++) {
                cBindingController.bindFc((listOfBindings.get(i)).getClientInterfaceName(),
                        (listOfBindings.get(i)).getServerInterface());
            }
        }
    }

    /**
     * Create an explorer for the specified component (the component requires an
     * explicit interface named fcAppl of type
     * org.objectweb.fractal.api.Component).
     *
     * @param component
     *            the component to explore
     * @throws ADLException :
     *             occurs when it is impossible to create the explorer
     * @throws NoSuchInterfaceException :
     *             occurs when it is impossible to create the explorer
     * @throws IllegalBindingException :
     *             occurs when it is impossible to create the explorer
     * @throws IllegalLifeCycleException :
     *             occurs when it is impossible to create the explorer
     * @throws IllegalContentException :
     *             occurs when it is impossible to create the explorer
     */
    public void createExplorer(final Component component) throws ADLException,
            NoSuchInterfaceException, IllegalContentException, IllegalLifeCycleException,
            IllegalBindingException {
        // create global composite
        Component globalComposite = this
                .createNewComponent(FractalHelper.PETALS_EXPLORER_COMPOSITE);

        ContentController ccg = Fractal.getContentController(globalComposite);
        // search the server component
        Component appli = this.getComponentByName(ccg, FractalHelper.EXPLORER_COMPOSITE);

        // Start of the petals explorer
        this.startComponent(appli);

        // add the component in the global composite
        this.addComponent(component, appli, null);

        // create explorer
        Component fractalExplorer = this
                .createNewComponent("org.objectweb.fractal.explorer.BasicFractalExplorerImpl");

        // create the bindings list of the server component to others components
        List<Binding> listOfBindingsExplorer = new ArrayList<Binding>();
        listOfBindingsExplorer.add(new Binding(FractalHelper.FCAPPL_BINDING, (Interface) appli
                .getFcInterface(FractalHelper.FCAPPL_BINDING)));
        this.addComponent(fractalExplorer, globalComposite, listOfBindingsExplorer);
    }

    /**
     *
     * @param rootContentController
     * @throws PetalsException
     * @throws NoSuchInterfaceException
     * @throws ADLException
     * @throws IllegalContentException
     * @throws IllegalLifeCycleException
     * @throws IllegalBindingException
     */
    public void setupDreamTransporter(ContentController rootContentController, int dreamPort)
            throws PetalsException {
        Component transporterComposite = getComponentByName(rootContentController,
                TRANSPORTER_COMPOSITE);
        ContentController transporterCompositeCC;
        try {

            // 1. set the port of the dream transporter
            // this MUST be done before the startFc of The PEtALS Composite
            transporterCompositeCC = Fractal.getContentController(transporterComposite);
            Component tcpIpProtocol = getComponentByName(transporterCompositeCC, "TCPIPProtocol");
            ContentController tcpIpProtocoCC = Fractal.getContentController(tcpIpProtocol);
            Component tcpIpProtocolImpl = getComponentByName(tcpIpProtocoCC, "Protocol");
            TCPIPProtocolImplAttributeController TCPProtocolAC = (TCPIPProtocolImplAttributeController) Fractal
                    .getAttributeController(tcpIpProtocolImpl);
            TCPProtocolAC.setPort(dreamPort);

        } catch (NoSuchInterfaceException e) {
            e.printStackTrace();
            String msg = "Error while configuring the transporter.";
            throw new PetalsException(msg, e);
        }
    }
}
