/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2007 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): Adrien LOUIS
 * --------------------------------------------------------------------------
 * $Id: R
 * --------------------------------------------------------------------------
 */

package org.ow2.petals.monitoring.router;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.jbi.component.context.ComponentContextImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.routing.Router;
import org.ow2.petals.jbi.messaging.routing.RoutingException;
import org.ow2.petals.util.LoggingUtil;

/**
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.1
 * @author alouis
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.jbi.messaging.routing.Router.class))

    //@Interface(name = "monitoring-service", signature = org.ow2.petals.monitoring.router.RouterMonitor.class)
public class RouterMonitorImpl implements Router, RouterMonitor {


    protected LoggingUtil log;

    @Monolog(name = "logger")
    protected Logger logger;

        private boolean monitoring = false;

    private Monitoring monitoring_util;

    @Requires(name = "router", signature = org.ow2.petals.jbi.messaging.routing.Router.class)
    private Router router;

    private boolean showMessageContent = false;

    // //////////////////////////////////////////
    // Router implementation
    // //////////////////////////////////////////

     /**
     * starts the monitoring with or without monitoring content of messages. If
     * monitoring is already started, just set the monitoring of messages
     * content.
     *
     * @param showMessageContent
     *            indicates if content of messages have to be monitored, true
     *            for monitor the content, false for ignoring it
     */
    public void activateMonitoring(boolean showMessageContent) {
        monitoring = true;
        this.showMessageContent = showMessageContent;
    }

    public void addComponent(String componentName) throws RoutingException {
        router.addComponent(componentName);
    }

    // //////////////////////////////////////////
    // Monitor implementation
    // //////////////////////////////////////////

    /**
     * Stops the monitoring of messages and their content.
     */
    public void deactivateMonitoring() {
        monitoring = false;
        showMessageContent = false;
    }


    /**
     * Indicates if content of messages have to be set in the messages monitored
     *
     * @return true if content of messages have to be in the messages, false if
     *         not
     */
    public boolean isMessageContentShown() {
        return showMessageContent;
    }

    public void showMessageContent(boolean showMessageContent) {
        this.showMessageContent = showMessageContent;
    }

    /**
     * Indicates if monitoring is started or not
     *
     * @return true if monitoring is started, false if not
     */
    public boolean isMonitoring() {
        return monitoring;
    }

    public MessageExchangeImpl receive(ComponentContextImpl source,
        long timeoutMS) throws RoutingException {

        MessageExchangeImpl exchange = null;

        exchange = router.receive(source, timeoutMS);

        if (monitoring && exchange != null) {
            report(exchange);
        }

        return exchange;
    }

    // ----------------------------------------------------------------------
    // Fractal LifecycleController implementation
    // ----------------------------------------------------------------------

    public void removeComponent(String componentName) throws RoutingException {
        router.removeComponent(componentName);
    }

    public void send(ComponentContextImpl source, MessageExchangeImpl exchange,
        long timeOut) throws RoutingException {

        if (monitoring && exchange != null) {
            report(exchange);
        }

        router.send(source, exchange, timeOut);
    }

    public MessageExchangeImpl sendSync(ComponentContextImpl source,
        MessageExchangeImpl exchange, long timeOut) throws RoutingException {

        if (monitoring && exchange != null) {
            report(exchange);
        }

        MessageExchangeImpl responseExchange = router.sendSync(source,
            exchange, timeOut);

        // report the response exchange
        if (monitoring && exchange != null) {
            report(responseExchange);
        }

        return responseExchange;
    }

    /**
     * Sets the monitoring object to a new Monitoring
     *
     * @param monitoring_util
     *            the Monitoring object to use
     */
    public void setMonitoring_util(Monitoring monitoring_util) {
        this.monitoring_util = monitoring_util;
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        log = new LoggingUtil(logger);
        log.call();
    }

    /**
     * catch information on the specified exchange
     *
     * @param exchange
     *            must be non null
     */
    private void report(MessageExchangeImpl exchange) {
        monitoring_util.addMessage(exchange.getExchangeId(), exchange,
            showMessageContent);
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {
        log.call();
    }
}
