/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ClassLoaderService.java 2948 2007-05-03 16:08:30Z chamerling $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.platform.classloader;

import java.net.URL;
import java.net.URLClassLoader;
import java.util.List;

import org.ow2.petals.PetalsException;

/**
 * 
 * @version $Rev: 2948 $ $Date: 2007-05-03 18:08:30 +0200 (jeu, 03 mai 2007) $
 * @since Petals 1.0
 * @author Adrien LOUIS, <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public interface ClassLoaderService {

    boolean containsClassLoader(String classLoaderId);

    /**
     * Must not return null.
     * 
     * @param baseUrls
     * @param classpathUrls
     * @param useParentFirst
     * @return
     * @throws PetalsException
     */
    URLClassLoader createClassLoader(URL[] baseUrls,
            List<String> classpathUrls, boolean useParentFirst)
            throws PetalsException;

    /**
     * 
     * @param classLoaderId
     * @param baseUrls
     * @param classpathUrls
     * @param useParentFirst
     * @param sharedLibrariesNameList
     * @return
     * @throws PetalsException
     */
    URLClassLoader createComponentClassLoader(String classLoaderId,
            URL[] baseUrls, List<String> classpathUrls, boolean useParentFirst,
            List<String> sharedLibrariesNameList) throws PetalsException;

    /**
     * Create a classloader for a sharedLibrary. This classLoader will be added
     * to the SharedLibrariesClassLoader.
     * 
     * @param baseUrls
     * @param classpathUrls
     * @param useParentFirst
     * @return
     * @throws PetalsException
     */
    URLClassLoader createSharedLibraryClassLoader(String classLoaderId,
            URL[] baseUrls, List<String> classpathUrls, boolean useParentFirst)
            throws PetalsException;

    /**
     * Remove the classLoader from the list. If the classLoader is a ClassLoader
     * for sharedLibraries, remove it from the SharedLibrariesClassLoader
     */
    void deleteClassLoader(String classLoaderId);
}
